@@ -422,9 +422,10 @@ static unsigned int reserve_eilvt_offset(int offset, unsigned int new)
 		if (vector && !eilvt_entry_is_changeable(vector, new))
 			/* may not change if vectors are different */
 			return rsvd;
-	} while (!atomic_try_cmpxchg(&eilvt_offsets[offset], &rsvd, new));
+		rsvd = atomic_cmpxchg(&eilvt_offsets[offset], rsvd, new);
+	} while (rsvd != new);
 
-	rsvd = new & ~APIC_EILVT_MASKED;
+	rsvd &= ~APIC_EILVT_MASKED;
 	if (rsvd && rsvd != vector)
 		pr_info("LVT offset %d assigned for vector 0x%02x\n",
 			offset, rsvd);
@@ -503,19 +504,7 @@ static int lapic_timer_shutdown(struct clock_event_device *evt)
 	v = apic_read(APIC_LVTT);
 	v |= (APIC_LVT_MASKED | LOCAL_TIMER_VECTOR);
 	apic_write(APIC_LVTT, v);
-
-	/*
-	 * Setting APIC_LVT_MASKED (above) should be enough to tell
-	 * the hardware that this timer will never fire. But AMD
-	 * erratum 411 and some Intel CPU behavior circa 2024 say
-	 * otherwise.  Time for belt and suspenders programming: mask
-	 * the timer _and_ zero the counter registers:
-	 */
-	if (v & APIC_LVT_TIMER_TSCDEADLINE)
-		wrmsrl(MSR_IA32_TSC_DEADLINE, 0);
-	else
-		apic_write(APIC_TMICT, 0);
-
+	apic_write(APIC_TMICT, 0);
 	return 0;
 }
 
@@ -1772,11 +1761,11 @@ static int x2apic_state;
 
 static bool x2apic_hw_locked(void)
 {
-	u64 x86_arch_cap_msr;
+	u64 ia32_cap;
 	u64 msr;
 
-	x86_arch_cap_msr = x86_read_arch_cap_msr();
-	if (x86_arch_cap_msr & ARCH_CAP_XAPIC_DISABLE) {
+	ia32_cap = x86_read_arch_cap_msr();
+	if (ia32_cap & ARCH_CAP_XAPIC_DISABLE) {
 		rdmsrl(MSR_IA32_XAPIC_DISABLE_STATUS, msr);
 		return (msr & LEGACY_XAPIC_DISABLED);
 	}
