@@ -270,13 +270,13 @@ EXPORT_SYMBOL(tty_termios_copy_hw);
  *	between the two termios structures, or a speed change is needed.
  */
 
-int tty_termios_hw_change(const struct ktermios *a, const struct ktermios *b)
+bool tty_termios_hw_change(const struct ktermios *a, const struct ktermios *b)
 {
 	if (a->c_ispeed != b->c_ispeed || a->c_ospeed != b->c_ospeed)
-		return 1;
+		return true;
 	if ((a->c_cflag ^ b->c_cflag) & ~(HUPCL | CREAD | CLOCAL))
-		return 1;
-	return 0;
+		return true;
+	return false;
 }
 EXPORT_SYMBOL(tty_termios_hw_change);
 
@@ -500,43 +500,22 @@ static int set_termios(struct tty_struct *tty, void __user *arg, int opt)
 	tmp_termios.c_ispeed = tty_termios_input_baud_rate(&tmp_termios);
 	tmp_termios.c_ospeed = tty_termios_baud_rate(&tmp_termios);
 
-	if (opt & (TERMIOS_FLUSH|TERMIOS_WAIT)) {
-retry_write_wait:
-		retval = wait_event_interruptible(tty->write_wait, !tty_chars_in_buffer(tty));
-		if (retval < 0)
-			return retval;
+	ld = tty_ldisc_ref(tty);
 
-		if (tty_write_lock(tty, false) < 0)
-			goto retry_write_wait;
-
-		/* Racing writer? */
-		if (tty_chars_in_buffer(tty)) {
-			tty_write_unlock(tty);
-			goto retry_write_wait;
-		}
-
-		ld = tty_ldisc_ref(tty);
-		if (ld != NULL) {
-			if ((opt & TERMIOS_FLUSH) && ld->ops->flush_buffer)
-				ld->ops->flush_buffer(tty);
-			tty_ldisc_deref(ld);
-		}
-
-		if ((opt & TERMIOS_WAIT) && tty->ops->wait_until_sent) {
-			tty->ops->wait_until_sent(tty, 0);
-			if (signal_pending(current)) {
-				tty_write_unlock(tty);
-				return -ERESTARTSYS;
-			}
-		}
-
-		tty_set_termios(tty, &tmp_termios);
-
-		tty_write_unlock(tty);
-	} else {
-		tty_set_termios(tty, &tmp_termios);
+	if (ld != NULL) {
+		if ((opt & TERMIOS_FLUSH) && ld->ops->flush_buffer)
+			ld->ops->flush_buffer(tty);
+		tty_ldisc_deref(ld);
 	}
 
+	if (opt & TERMIOS_WAIT) {
+		tty_wait_until_sent(tty, 0);
+		if (signal_pending(current))
+			return -ERESTARTSYS;
+	}
+
+	tty_set_termios(tty, &tmp_termios);
+
 	/* FIXME: Arguably if tmp_termios == tty->termios AND the
 	   actual requested termios was not tmp_termios then we may
 	   want to return an error as no user requested change has
@@ -859,7 +838,7 @@ int tty_mode_ioctl(struct tty_struct *tty, unsigned int cmd, unsigned long arg)
 			ret = -EFAULT;
 		return ret;
 	case TIOCSLCKTRMIOS:
-		if (!checkpoint_restore_ns_capable(&init_user_ns))
+		if (!capable(CAP_SYS_ADMIN))
 			return -EPERM;
 		copy_termios_locked(real_tty, &kterm);
 		if (user_termios_to_kernel_termios(&kterm,
@@ -876,7 +855,7 @@ int tty_mode_ioctl(struct tty_struct *tty, unsigned int cmd, unsigned long arg)
 			ret = -EFAULT;
 		return ret;
 	case TIOCSLCKTRMIOS:
-		if (!checkpoint_restore_ns_capable(&init_user_ns))
+		if (!capable(CAP_SYS_ADMIN))
 			return -EPERM;
 		copy_termios_locked(real_tty, &kterm);
 		if (user_termios_to_kernel_termios_1(&kterm,
